package de.ozgcloud.apilib.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.user.grpc.userprofile.UserProfileServiceGrpc.UserProfileServiceBlockingStub;
import de.ozgcloud.user.userprofile.GrpcGetUserProfileRequest;

class GrpcOzgCloudUserProfileServiceTest {

	@InjectMocks
	private GrpcOzgCloudUserProfileService service;

	@Mock
	private UserProfileServiceBlockingStub userProfileServiceStub;
	@Mock
	private UserProfileMapper mapper;

	@Mock
	private OzgCloudCallContextProvider contextProvider;

	@BeforeEach
	void prepareStubMock() {
		when(userProfileServiceStub.withInterceptors(any())).thenReturn(userProfileServiceStub);
	}

	@Nested
	class TestGetById {

		@Captor
		private ArgumentCaptor<GrpcGetUserProfileRequest> requestCaptor;

		private OzgCloudUserProfile mappedUserProfile = OzgCloudUserProfileTestFactory.create();

		@BeforeEach
		void init() {
			when(userProfileServiceStub.getById(any())).thenReturn(
					GrpcGetUserProfileResponseTestFactory.create());
			when(mapper.mapFromGrpc(any())).thenReturn(mappedUserProfile);
		}

		@Test
		void shouldCallStub() {
			service.getById(OzgCloudUserProfileTestFactory.ID);

			verify(userProfileServiceStub).getById(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getUserId()).isEqualTo(
					OzgCloudUserProfileTestFactory.ID.toString());
		}

		@Test
		void shouldCallMapper() {
			service.getById(OzgCloudUserProfileTestFactory.ID);

			verify(mapper).mapFromGrpc(any());
		}

		@Test
		void shouldReturnResult() {
			var result = service.getById(OzgCloudUserProfileTestFactory.ID);

			assertThat(result).isSameAs(mappedUserProfile);
		}

		@Test
		void shouldAddInterceptor() {
			service.getById(OzgCloudUserProfileTestFactory.ID);

			verify(userProfileServiceStub).withInterceptors(notNull());
		}
	}

}
